"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.javaLikeGenerator = exports.options = exports.identifierCamelCase = exports.classCamelCase = void 0;
const types_1 = require("@openapi-generator-plus/types");
const generator_common_1 = require("@openapi-generator-plus/generator-common");
const change_case_1 = require("change-case");
const generator_common_2 = require("@openapi-generator-plus/generator-common");
/** Returns the string converted to a string that is safe as an identifier in java-like languages */
function identifierSafe(value) {
    if (typeof value !== 'string') {
        throw new Error(`identifierSafe called with non-string: ${typeof value} (${value})`);
    }
    /* Add a prefix if the identifier starts with illegal characters */
    if (value.match(/^[^a-zA-Z_]/)) {
        value = `_${value}`;
    }
    /* Convert any illegal characters to underscores */
    value = value.replace(/[^a-zA-Z0-9_]/g, '_');
    return value;
}
/**
 * Camel case and capitalize suitable for a class name. Doesn't change existing
 * capitalization in the value.
 * e.g. "FAQSection" remains "FAQSection", and "faqSection" will become "FaqSection"
 * @param value string to be turned into a class name
 */
function classCamelCase(value) {
    return identifierSafe(generator_common_1.pascalCase(identifierSafe(value)));
}
exports.classCamelCase = classCamelCase;
function identifierCamelCase(value) {
    return identifierSafe(generator_common_1.camelCase(identifierSafe(value)));
}
exports.identifierCamelCase = identifierCamelCase;
function options(config, context) {
    const result = {
        modelClassPrefix: config.modelClassPrefix,
        constantStyle: config.constantStyle || context.defaultConstantStyle,
    };
    return result;
}
exports.options = options;
function javaLikeGenerator(config, context) {
    const generatorOptions = options(config, context);
    const cg = generator_common_2.commonGenerator(config, context);
    return {
        toClassName: (name) => {
            return classCamelCase(name);
        },
        toIdentifier: (name) => {
            let result = identifierCamelCase(name);
            const reservedWords = context.reservedWords ? context.reservedWords() : [];
            while (reservedWords.indexOf(result) !== -1) {
                result = identifierCamelCase(`a_${name}`);
            }
            return result;
        },
        toConstantName: (name) => {
            const constantStyle = generatorOptions.constantStyle;
            switch (constantStyle) {
                case "allCaps" /* allCaps */:
                    return identifierSafe(change_case_1.constantCase(identifierSafe(name)).replace(/_/g, ''));
                case "camelCase" /* camelCase */:
                    return identifierCamelCase(name);
                case "snake" /* allCapsSnake */:
                    return identifierSafe(change_case_1.constantCase(identifierSafe(name)));
                case "pascalCase" /* pascalCase */:
                    return identifierSafe(generator_common_1.pascalCase(identifierSafe(name)));
                default:
                    throw new Error(`Invalid valid for constantStyle: ${constantStyle}`);
            }
        },
        toSchemaName: (name, options) => {
            let result = cg.toSchemaName(name, options);
            result = context.generator().toClassName(result);
            if ((options.schemaType === types_1.CodegenSchemaType.OBJECT || options.schemaType === types_1.CodegenSchemaType.ENUM) && generatorOptions.modelClassPrefix) {
                result = generatorOptions.modelClassPrefix + result;
            }
            return result;
        },
        toSuggestedSchemaName: (name, options) => {
            if (options.schemaType === types_1.CodegenSchemaType.ENUM) {
                name = `${name}_enum`;
            }
            return cg.toSuggestedSchemaName(name, options);
        },
        toOperationGroupName: (name) => {
            return context.generator().toClassName(name);
        },
    };
}
exports.javaLikeGenerator = javaLikeGenerator;
